# authentic2-auth-kerberos - Kerberos plugin for authentic2
# Copyright (C) 2010-2019 Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mock

import pytest

from authentic2.custom_user.models import User


@pytest.fixture(autouse=True)
def kerberos(monkeypatch):
    import gssapi.exceptions

    class MockKerberos:
        principal = 'user@REALM'

        exceptions = gssapi.exceptions
        Credentials = mock.Mock()
        server_ctx = mock.Mock()
        server_ctx.step.return_value = b'outtoken'
        server_ctx.complete = True
        server_ctx.initiator_name = principal

        SecurityContext = mock.Mock(return_value=server_ctx)
    monkeypatch.setattr('django_kerberos.views.gssapi', MockKerberos)
    return MockKerberos


def test_default(settings, app, db, kerberos):
    settings.A2_AUTH_KERBEROS_DJANGO_BACKEND = True

    user = User.objects.create(username=kerberos.principal)
    assert 'a2_kerberos_ok' not in app.cookies
    response = app.get('/login/')
    assert 'login-kerberos' in response.text
    assert 'autologin' not in response.text
    response = response.forms['kerberos-form'].submit(name='login-kerberos')
    assert response.location == '/accounts/kerberos/login/'
    response = response.follow(headers={'Authorization': 'Negotiate yyyy'})
    assert app.cookies['a2_kerberos_ok'] == '1'
    assert app.session['_auth_user_id'] == str(user.id)
    assert response.location == '/'

    # logout
    app.session.flush()
    response = app.get('/login/')
    assert 'login-kerberos' in response.text
    assert 'autologin' in response.text
